Attribute VB_Name = "mDialogHook"
'Common Dialog Hooking Module
'
'Provides the hook procedure to subclass the common dialog.
'
'(c) 2000 by Joe Hart (bghost@ti.cz)
'
'Thanks to Bruce McKinney and Steve McMahon for information about Common Dialogs
Option Explicit

Private Declare Function SendMessageStr Lib "user32" Alias "SendMessageA" (ByVal hwnd As Long, ByVal wMsg As Long, ByVal wParam As Long, ByVal lParam As String) As Long
Private Declare Sub CopyMemory Lib "kernel32" Alias "RtlMoveMemory" (lpvDest As Any, lpvSource As Any, ByVal cbCopy As Long)
Private Declare Function GetParent Lib "user32" (ByVal hwnd As Long) As Long

' Messages which can be sent to the standard dialog elements
Private Const WM_USER = &H400
Private Const CDM_FIRST = (WM_USER + 100)
Private Const CDM_LAST = (WM_USER + 200)
Private Const CDM_GETSPEC = (CDM_FIRST + &H0)
Private Const CDM_GETFILEPATH = (CDM_FIRST + &H1)
Private Const CDM_GETFOLDERPATH = (CDM_FIRST + &H2)
Private Const CDM_GETFOLDERIDLIST = (CDM_FIRST + &H3)
Private Const CDM_SETCONTROLTEXT = (CDM_FIRST + &H4)
Private Const CDM_HIDECONTROL = (CDM_FIRST + &H5)
Private Const CDM_SETDEFEXT = (CDM_FIRST + &H6)

' IDs for standard common dialog controls
Private Const ID_OPEN = &H1  'Open or Save button
Private Const ID_CANCEL = &H2 'Cancel Button
Private Const ID_HELP = &H40E 'Help Button
Private Const ID_READONLY = &H410 'Read-only check box
Private Const ID_FILETYPELABEL = &H441 'Files of type label
Private Const ID_FILELABEL = &H442 'File name label
Private Const ID_FOLDERLABEL = &H443 'Look in label
Private Const ID_LIST = &H461 'Parent of file list
Private Const ID_FORMAT = &H470 'File type combo box
Private Const ID_FOLDER = &H471 'Folder combo box
Private Const ID_FILETEXT = &H480 'File name text box

Private m_cHookedDialog As Long

Property Let HookedDialog(ByRef cThis As cNewDialog)
    'Set cHookedDialog = cThis
    m_cHookedDialog = ObjPtr(cThis)
End Property
Property Get HookedDialog() As cNewDialog
   Dim oThis As cNewDialog
   If (m_cHookedDialog <> 0) Then
      ' Turn the pointer into an illegal, uncounted interface
      CopyMemory oThis, m_cHookedDialog, 4
      ' Do NOT hit the End button here! You will crash!
      ' Assign to legal reference
      Set HookedDialog = oThis
      ' Still do NOT hit the End button here! You will still crash!
      ' Destroy the illegal reference
      CopyMemory oThis, 0&, 4
   End If
End Property
Public Sub ClearHookedDialog()
    m_cHookedDialog = 0
End Sub

Public Function DialogHookFunction(ByVal hDlg As Long, ByVal msg As Long, ByVal wParam As Long, ByVal lParam As Long) As Long
   Dim ComDlg As cNewDialog
   Set ComDlg = HookedDialog
   If Not (ComDlg Is Nothing) Then 'just to make sure the class was created properly...
      DialogHookFunction = ComDlg.DialogHook(hDlg, msg, wParam, lParam)
   End If
End Function

Public Property Get GetComDlgFileName(ByVal hDlg As Long) As String
   Dim sBuf As String
   Dim Pos As Long
   Dim hwnd As Long
   hwnd = GetParent(hDlg)
   sBuf = String$(260, 0)
   SendMessageStr hwnd, CDM_GETFILEPATH, 260, sBuf
   GetComDlgFileName = NullTrim(sBuf)
End Property

Public Function NullTrim(s) As String
'convert a null terminated string to standard vb string, deleting any leading or trailing spaces
     Dim i As Integer
     i = InStr(s, vbNullChar)
     If i > 0 Then s = Left$(s, i - 1)
     s = Trim$(s)
     NullTrim = s
End Function

